; milliGun/theme.jl

;; Copyright (C) 1999 Ryan Lovett <ryan@ocf.berkeley.edu>
;; Copyright (C) 2000 Pierre Baillargeon <pierreba@globetrotter.net>

;; This theme is derived from microGUI, with customizable amount of blue,
;; centered title, close button alone on the left, nicer transient windows,
;; and bigger image sizes (x1.5) for a nice 1600x1200 style.

(defgroup milliGun "milliGun Theme"
  :group appearance)

(defcustom milliGun:percent-blue 90
  "Percent of unused title in blue."
  :group (appearance milliGun)
  :type number
  :range (0 . 100))

(defcustom milliGun:percent-green 50
  "Percent of unused transient title in green."
  :group (appearance milliGun)
  :type number
  :range (0 . 100))

(let*


    ;; Update window title pixel length
    ((title-width
      (lambda (w)
	(let
	    ((w-width (car (window-dimensions w))))
	  (max 0 (min (- w-width 100) (text-width (window-name w)))))))

     (w-width (lambda (w) (car (window-dimensions w))))

     (w-height (lambda (w) (cdr (window-dimensions w))))

     (blue-width   (lambda (w) (max 0 (/ (* (/ (- (w-width w)  (+ (title-width w) 168) ) 2) milliGun:percent-blue) 100))))

     (green-height (lambda (w) (max 0 (/ (*    (- (w-height w)                     56  )    milliGun:percent-green) 100))))

     ;; called after the WM_NAME property changes
     (update-text-width
      (lambda (w)
	(if (eq (window-get w 'current-frame-style) 'milliGun)
	    (rebuild-frame w))))
     
     ;; 2x6
     (bottom-images (make-image "bottom.png"))

     ;; 6x40
     (top-left-images (list (make-image "top_left_inactive.png")
			    (make-image "top_left.png")))
     (top-left-shaped-images (list (make-image "top_left_inactive_s.png")
				   (make-image "top_left_s.png")))

     ;; 19x28
     (top-blue-images (list (make-image "top_blue_inactive.png")
			    (make-image "top_blue.png")))

     ;; 18x28
     (top-curves-images (list (make-image "top_curves_inactive.png")
			      (make-image "top_curves.png")))

     ;; 18x28
     (top-right-curves-images (list (make-image "top_right_curves_inactive.png")
			      (make-image "top_right_curves.png")))

     ;; 19x28
     (top-grey-images (make-image "top_grey.png"))

     ;; 6x40
     (top-right-images (list (make-image "top_right_inactive.png")
                             (make-image "top_right.png")))
     (top-right-shaped-images (list (make-image "top_right_inactive_s.png")
				    (make-image "top_right_s.png")))

     ;; 6x6
     (bottom-left-images (make-image "bl.png"))

     ;; 6x6
     (bottom-right-images (make-image "br.png"))

     ;; 25x28
     (close-images (list (make-image "close_normal.png")
			 (make-image "close_active.png") nil
			 (make-image "close_clicked.png")))

     ;; 25x28
     (maximize-images (list (make-image "maximize_normal.png")
			    (make-image "maximize_active.png") nil
			    (make-image "maximize_clicked.png")))

     ;; 25x28
     (iconify-images (list (make-image "minimize_normal.png")
			   (make-image "minimize_active.png") nil
			   (make-image "minimize_clicked.png")))

     ;; 25x28
     (menu-images (list (make-image "menu_normal.png")
			(make-image "menu_active.png") nil
			(make-image "menu_clicked.png")))

     ;; 6x19
     (left-images (make-image "left.png"))

     ;; 6x19
     (right-images (make-image "right.png"))

     (top-images (flip-image-diagonally (copy-image left-images)))

     ;; 27x25
     (t-close-images (list (make-image "t_close_normal.png")
			   (make-image "t_close_active.png") nil
			   (make-image "t_close_clicked.png")))
     ;; 18x28
     (t-right-curves-images (list (make-image "t_curves_inactive.png")
			      (make-image "t_curves.png")))

     (t-right-green-images (list (make-image "t_right_green_inactive.png")
			      (make-image "t_right_green.png")))
     ;; 5x19
     (t-left-images (make-image "t_left.png"))

     ;; 27x10
     (t-right-images (make-image "t_right.png"))

     ;; 28x5
     (t-top-images (make-image "t_top.png"))

     ;; 28x5
     (t-bottom-images (make-image "t_bottom.png"))

     ;; 4x4
     (t-top-left-images (make-image "t_top_left.png"))

     ;; 4x4
     (t-bottom-left-images (make-image "t_bottom_left.png"))

     ;; 25x4
     (t-top-right-images (list (make-image "t_top_right_inactive.png")
                               (make-image "t_top_right.png")))
     (t-top-right-shaped-images (list (make-image "t_top_right_inactive_s.png")
				       (make-image "t_top_right_s.png")))

     ;; 25x4
     (t-bottom-right-images (make-image "t_bottom_right.png"))

     ;; frame layout

     (frame `(((background . ,top-left-images)
	       (left-edge . -6)
	       (top-edge . -28)
	       (class . top-left-corner))

	      ;; delete button
	      ((background . ,close-images )
	       (top-edge . -28)
	       (left-edge . 0)
	       (class . close-button))

	      ;; menu button
	      ((background . ,menu-images)
	       (top-edge . -28)
	       (right-edge . 0)
	       (class . menu-button))

	      ;; top blue
	      ((background . ,top-blue-images)
	       (top-edge . -28)
	       (left-edge . 28)
	       (width . ,(lambda (w) (blue-width w)))
	       (class . title))

	      ;; top curves
	      ((background . ,top-curves-images)
	       (left-edge . ,(lambda (w) (+ (blue-width w) 28)))
	       (top-edge . -28)
	       (class . title))

	      ;; top grey
	      ((background . ,top-grey-images)
	       (foreground . "black")
	       (text . ,window-name)
	       (x-justify . center)
	       (y-justify . top)
	       (top-edge . -28)
	       (left-edge . ,(lambda (w) (+ (blue-width w) 46)))
	       (right-edge . ,(lambda (w) (+ (blue-width w) 102)))
	       (class . title))

	      ;; left border
	      ((background . ,left-images)
	       (left-edge . -6)
	       (top-edge . 8)
	       (bottom-edge . 0)
	       (class . left-border))

	      ;; top-right corner
	      ((background . ,top-right-images)
	       (right-edge . -6)
	       (top-edge . -28)
	       (class . top-right-corner))

	      ;; right border
	      ((background . ,right-images)
	       (right-edge . -6)
	       (top-edge . 8)
	       (bottom-edge . 0)
	       (class . right-border))

	      ;; bottom border
	      ((background . ,bottom-images)
	       (left-edge . 0)
	       (right-edge . 0)
	       (bottom-edge . -6)
	       (class . bottom-border))

	      ;; bottom-left corner
	      ((background . ,bottom-left-images)
	       (left-edge . -6)
	       (bottom-edge . -6)
	       (class . bottom-left-corner))

	      ;; bottom-right corner
	      ((background . ,bottom-right-images)
	       (right-edge . -6)
	       (bottom-edge . -6)
	       (class . bottom-right-corner))

	      ;; top right curves
	      ((background . ,top-right-curves-images)
	       (right-edge . ,(lambda (w) (+ (blue-width w) 84)))
	       (top-edge . -28)
	       (class . title))

	      ;; top blue
	      ((background . ,top-blue-images)
	       (top-edge . -28)
	       (right-edge . 84)
	       (width . ,(lambda (w) (blue-width w)))
	       (class . title))

	      ;; iconify button
	      ((background . ,iconify-images)
	       (right-edge . 56)
	       (top-edge . -28)
	       (class . iconify-button))

	      ;; maximize button
	      ((background . ,maximize-images)
	       (right-edge . 28)
	       (top-edge . -28)
	       (class . maximize-button))
					 ))

       (shaped-frame `(
		       ((background . ,top-left-shaped-images)
			(left-edge . -6)
			(top-edge . -28)
			(height . 28)
			(class . top-left-corner))

		       ;; delete button
		       ((background . ,close-images )
			(top-edge . -28)
			(left-edge . 0)
			(class . close-button))

		       ;; menu button
		       ((background . ,menu-images)
			(top-edge . -28)
			(right-edge . 0)
			(class . menu-button))

		       ;; top blue
		       ((background . ,top-blue-images)
			(top-edge . -28)
			(left-edge . 28)
			(width . ,(lambda (w) (blue-width w)))
			(class . title))

		       ;; top curves
		       ((background . ,top-curves-images)
			(left-edge . ,(lambda (w) (+ (blue-width w) 28)))
			(top-edge . -28)
			(class . title))

		       ;; top grey
		       ((background . ,top-grey-images)
			(foreground . "black")
			(text . ,window-name)
			(x-justify . center)
			(y-justify . top)
			(top-edge . -28)
			(left-edge . ,(lambda (w) (+ (blue-width w) 46)))
			(right-edge . ,(lambda (w) (+ (blue-width w) 102)))
			(class . title))

		       ;; top-right corner
		       ((background . ,top-right-shaped-images)
			(right-edge . -6)
			(top-edge . -28)
			(height . 28)
			(class . top-right-corner))

		       ;; top right curves
		       ((background . ,top-right-curves-images)
			(right-edge . ,(lambda (w) (+ (blue-width w) 84)))
			(top-edge . -28)
			(class . title))

		       ;; top blue
		       ((background . ,top-blue-images)
			(top-edge . -28)
			(right-edge . 84)
			(width . ,(lambda (w) (blue-width w)))
			(class . title))

		       ;; iconify button
		       ((background . ,iconify-images)
			(right-edge . 56)
			(top-edge . -28)
			(class . iconify-button))

		       ;; maximize button
		       ((background . ,maximize-images)
			(right-edge . 28)
			(top-edge . -28)
			(class . maximize-button))
						  ))

       (transient-frame `(((background . ,t-top-left-images)
			   (left-edge . -5)
			   (top-edge . -5)
			   (class . top-left-corner))

			  ;;top-right corner
			  ((background . ,t-top-right-images)
			   (right-edge . -27)
			   (top-edge . -5)
			   (class . top-right-corner))

			  ;;title border
			  ((background . ,t-top-images)
			   (left-edge . -1)
			   (right-edge . 8)
			   (top-edge . -5)
			   (class . top-border))

			  ;; left border
			  ((background . ,t-left-images)
			   (left-edge . -5)
			   (top-edge . -1)
			   (bottom-edge . -1)
			   (class . left-border))

			  ;; right border
			  ((background . ,t-right-images)
			   (right-edge . -27)
			   (top-edge . 1)
			   (bottom-edge . -1)
			   (class . title))

			  ;; bottom border
			  ((background . ,t-bottom-images)
			   (left-edge . -1)
			   (right-edge . -2)
			   (bottom-edge . -5)
			   (class . bottom-border))

			  ;; bottom-left corner
			  ((background . ,t-bottom-left-images)
			   (left-edge . -5)
			   (bottom-edge . -5)
			   (class . bottom-left-corner))

			  ;; bottom-right corner
			  ((background . ,t-bottom-right-images)
			   (right-edge . -27)
			   (bottom-edge . -5)
			   (class . bottom-right-corner))

			  ;; delete button
			  ((background . ,t-close-images)
			   (right-edge . -27)
			   (top-edge . 0)
			   (class . close-button))

			  ;; green
			  ((background . ,t-right-green-images)
			   (right-edge . -27)
			   (top-edge . 27)
			   (height . ,(lambda (w) (green-height w)))
			   (class . title))

			  ;; curves
			  ((background . ,t-right-curves-images)
			   (right-edge . -27)
			   (top-edge . ,(lambda (w) (+ 27 (green-height w))))
			   (class . title))
						  ))

       (shaped-transient-frame `(((background . ,t-top-right-shaped-images)
				  (right-edge . -32)
				  (top-edge . -5)
				  (class . top-right-corner))

				 ;; right border
				 ((background . ,t-right-images)
				  (right-edge . -32)
				  (top-edge . 1)
				  (bottom-edge . -1)
				  (class . title))

				 ;; bottom-right corner
				 ((background . ,t-bottom-right-images)
				  (right-edge . -32)
				  (bottom-edge . -5)
				  (class . bottom-right-corner))

				 ;; delete button
				 ((background . ,t-close-images)
				  (right-edge . -32)
				  (top-edge . 0)
				  (class . close-button))

				 ;; green
				 ((background . ,t-right-green-images)
				  (right-edge . -32)
				  (top-edge . 27)
				  (height . ,(lambda (w) (green-height w)))
				  (class . title))

				 ;; curves
				 ((background . ,t-right-curves-images)
				  (right-edge . -32)
				  (top-edge . ,(lambda (w) (+ 27 (green-height w))))
				  (class . title))
						   ))
						     )
  
  (add-frame-style 'milliGun
		   (lambda (w type)
		     (cond ((eq type 'shaped)
			    shaped-frame)
			   ((eq type 'transient)
			    transient-frame)
			   ((eq type 'shaped-transient)
			    shaped-transient-frame)
			   ((eq type 'unframed)
			    nil-frame)
			   (t
			    frame))))

  (call-after-property-changed 'WM_NAME update-text-width)

  (mapc (lambda (sym)
	  (custom-set-property sym ':after-set (lambda() (rebuild-frames-with-style 'milliGun))))
	'(milliGun:percent-blue milliGun:percent-green))
  )
